/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WPROBLEM_H
#define WPROBLEM_H

#include "dart.h"
#include "wObject.h"
#include "wF1Ct.h"
#include <memory>
#include <iostream>
#include <vector>
#include <Eigen/Dense>

namespace dart
{

/**
 * @brief Formulation of the problem
 *
 * Contains freestream definition, reference values and physical groups
 * @authors Adrien Crovato
 */
class DART_API Problem : public fwk::wSharedObject
{
public:
    std::shared_ptr<tbox::MshData> msh; ///< Mesh structure
    int nDim;                           ///< Problem dimension
    // Freestream
    double alpha; ///< Angle of attack
    double beta;  ///< Angle of sideslip
    double M_inf; ///< Mach number
    // Reference values
    double S_ref;          ///< Reference surface
    double c_ref;          ///< Reference chord
    Eigen::Vector3d x_ref; ///< Reference center point (for moment computation)
#ifndef SWIG
    // Loads direction
    F1CtDrag dirD; // drag
    F1CtSide dirS; // sideforce
    F1CtLift dirL; // lift
    // Physical groups
    std::shared_ptr<Fluid> fluid;                  ///< Fluid air
    std::vector<std::shared_ptr<Body>> bodies;     ///< Bodies immersed in Fluid
    std::shared_ptr<Initial> iIC;                  ///< Initial condition
    std::vector<std::shared_ptr<Dirichlet>> dBCs;  ///< Dirichlet boundary conditions
    std::vector<std::shared_ptr<Freestream>> fBCs; ///< Freestream boundary conditions
    std::vector<std::shared_ptr<Wake>> wBCs;       ///< Wake boundary condition
    std::vector<std::shared_ptr<Kutta>> kSCs;      ///< Kutta condition
    std::vector<std::shared_ptr<Blowing>> bBCs;    ///< Blowing (transpiration) boundary condition
    // Observers
    std::vector<fwk::Observer *> obs; ///< classes depending on problem variables
#endif

    Problem(std::shared_ptr<tbox::MshData> _msh, int dim, double aoa, double aos,
            double minf, double sref, double cref, double xref, double yref,
            double zref);
    virtual ~Problem() { std::cout << "~Problem()\n"; }

    void set(std::shared_ptr<Fluid> f);
    void add(std::shared_ptr<Body> b);
    void set(std::shared_ptr<Initial> i);
    void add(std::shared_ptr<Dirichlet> d);
    void add(std::shared_ptr<Freestream> f);
    void add(std::shared_ptr<Wake> w);
    void add(std::shared_ptr<Kutta> k);
    void add(std::shared_ptr<Blowing> b);
    void update(double aoa);

#ifndef SWIG
    void check() const;
    void pin();
    void initElems();
    void initGradElems();
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace dart

#endif //WPROBLEM_H
